import { isEmpty } from 'lodash'

type Serverless = {
  variables: Variables
}
type Variables = {
  renderMatches: () => any
}
type Options = {}

class EnvCheckPlugin {
  serverless: Serverless

  constructor(serverless: Serverless, options: Options) {
    this.serverless = serverless

    this.serverless.variables.renderMatches = function renderMatches(
      value: any,
      matches: any,
      results: any,
    ) {
      let result = value
      let missingVars: string[] = []
      for (let i = 0; i < matches.length; i += 1) {
        if (
          results[i] === null ||
          typeof results[i] === 'undefined' ||
          (typeof results[i] === 'object' && isEmpty(results[i]))
        ) {
          this.warnIfNotFound(matches[i].variable, results[i]);
          missingVars = [...missingVars, matches[i].variable]
        }

        result = this.populateVariable(result, matches[i].match, results[i])
      }

      if (missingVars.length > 0) {
        throw new this.serverless.classes.Error(
          `Could not resolve variable(s): ${missingVars.join(', ')}`,
        )
      }

      return result
    }.bind(this.serverless.variables)
  }
}

export = EnvCheckPlugin
